/**
 *
 * Openutils magnolia tasks (http://www.openmindlab.com/lab/products/mgnltasks.html)
 * Copyright(C) 2008-2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package it.openutils.mgnltasks;

import info.magnolia.cms.core.SystemProperty;
import info.magnolia.module.DefaultModuleVersionHandler;
import info.magnolia.module.InstallContext;
import info.magnolia.module.delta.Delta;
import info.magnolia.module.delta.DeltaBuilder;
import info.magnolia.module.model.Version;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.ObjectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * <p>
 * A base module version handler that re-bootstrap its configuration each time the version number changes. Can be used
 * as is or subclasses (usually overriding getStartupTasks() for configuration tweaks).
 * </p>
 * <p>
 * All the bootstrap files placed in "/mgnl-bootstrap/(modulename)" will be reloaded every time the version number
 * changes, while xml files placed in "/mgnl-bootstrap/(modulename)-nooverwrite" will be bootstrapped only if the node
 * is not already existing in the repository.
 * </p>
 * @author fgiust
 * @version $Id: SimpleModuleVersionHandler.java 1830 2010-02-07 10:16:00Z fgiust $
 */
public class SimpleModuleVersionHandler extends DefaultModuleVersionHandler
{

    /**
     * Logger.
     */
    protected Logger log = LoggerFactory.getLogger(getClass());

    /**
     * Repositories to exclude from bootstrap
     */
    protected List<String> excludedRepositoriesFromBootstrap = new ArrayList<String>();

    /**
     *
     */
    public SimpleModuleVersionHandler()
    {
        super();
        excludedRepositoriesFromBootstrap.add("website");
    }

    /**
     * Add a repository to bootstrap exclusion list
     * @param repository repository to exclude from bootstrap
     */
    public void addExcludeRepositoryFromBootstrap(String repository)
    {
        excludedRepositoriesFromBootstrap.add(repository);
    }

    /**
     * Remove a repository from bootstrap exclusion list
     * @param repository repository to enable for bootstrap
     */
    public void removeExcludeRepositoryFromBootstrap(String repository)
    {
        excludedRepositoriesFromBootstrap.remove(repository);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<Delta> getDeltas(InstallContext ctx, Version from)
    {
        if (from == null)
        {
            log.info("Actual version not set");
            List<Delta> deltas = super.getDeltas(ctx, from);

            // adding the "-nooverwrite" in addiction to the standard bootstrap directory
            deltas.get(0).getTasks().add(
                new BootstrapMissingNodesTask(ctx.getCurrentModuleDefinition().getName() + "-nooverwrite"));

            return deltas;
        }

        String modulename = ctx.getCurrentModuleDefinition().getName();

        Version to = ctx.getCurrentModuleDefinition().getVersion();

        List<Delta> deltas = new ArrayList<Delta>();

        if (SystemProperty.getBooleanProperty(modulename + ".update.disabled"))
        {
            return deltas;
        }

        if (!to.isEquivalent(from)
            || "${project.version}".equals(ObjectUtils.toString(from))
            || "SNAPSHOT".equals(from.getClassifier()))
        {
            log.info("Updating from version {}", from);

            Delta delta = DeltaBuilder.update(to, "Update to current version");
            delta.getTasks().add(new ModuleConfigBootstrapTask(modulename, excludedRepositoriesFromBootstrap));
            delta.getTasks().add(new BootstrapMissingNodesTask(modulename + "-nooverwrite"));

            delta.getTasks().add(new UpdateModuleVersionTask());

            deltas.add(delta);
        }
        else
        {
            log.info("Version {} already installed, no update tasks to run", from);
        }

        return deltas;
    }
}
